/* SPDX-License-Identifier: GPL-3.0-or-later */

.wy-side-nav-search {
    background-color: #b92929; /* RED FOR DEVELOPERS :-) */
}
.wy-side-nav-search input[type=text] {
    border-color: #a42424;
}
.wy-side-nav-search div.version {
    color: hsla(0,0%,100%,0.4);
}
.wy-nav-top {
    background-color: #b92929;
}

.wy-nav-content-wrap a {
    color: #b92929;
}
.wy-nav-content-wrap a:hover {
    color: #d13030;
}
.wy-nav-content-wrap a:visited {
    color: #780001;
}
.wy-nav-content-wrap a:visited:hover {
    color: #d13030;
}

.wy-menu-vertical p.caption {
    color: #d95555;
}
